package com.psychobit.autodisc;

import java.util.ArrayList;
import java.util.Random;

import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.block.Jukebox;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;


/**
 * AutoDisc allows a jukebox to play a random music disc
 * every interval. The interval and the location of the jukebox are set
 * via a config file, or in-game commands.
 * @author psychobit
 *
 */
public class AutoDisc  extends JavaPlugin
{
	
	/**
	 * Scheduled repeating task
	 */
	private int _thread;
	
	/**
	 * Start the task to play a disc
	 */
	public void onEnable()
	{
		this.startTask();
	}
	
	/**
	 * Kill the task when we shut down
	 * Prevents multiple running tasks
	 */
	public void onDisable()
	{
     	this.stopTask();
	}
	
	/**
	 * Create the scheduled repeating task
	 */
	public void startTask()
	{
		final AutoDisc plugin = this;
        this._thread = this.getServer().getScheduler().scheduleSyncRepeatingTask( this, new Runnable() {
            public void run() { plugin.playDisc(); }
        }, 20L, plugin.getConfig().getLong( "delay", 4000L ) );
	}
	
	/**
	 * Stop the task from running
	 */
	public void stopTask()
	{
		this.getServer().getScheduler().cancelTask( this._thread );
	}
	
	
	/**
	 * Have the jukebox play a random disc
	 */
	public void playDisc()
	{
		// Get the block location and check that it's a jukebox
    	int x = this.getConfig().getInt( "coords.x" );
    	int y = this.getConfig().getInt( "coords.y" );
    	int z = this.getConfig().getInt( "coords.z" );
    	Block block = this.getServer().getWorlds().get(0).getBlockAt(x, y, z);
    	if ( !( block.getState() instanceof Jukebox ) )
    	{
    		System.out.println( "[AutoDisc] No Jukebox Found at ( " + x + "," + y + "," + z + " ) - " + block.getState().getTypeId() );
    		return;
    	}
    	Jukebox jb = null;
    	try {
    		jb = ( Jukebox ) block.getState();
    	} catch( ClassCastException e ) {
    		System.out.println( "[AutoDisc] Error casting jukebox at ( " + x + "," + y + "," + z + " )" );
			return;
		}
    	
    	// List of all discs
    	ArrayList<Material> Discs = new ArrayList<Material>();
    	Discs.add( Material.GOLD_RECORD );
    	Discs.add( Material.GREEN_RECORD );
    	Discs.add( Material.RECORD_10 );
    	Discs.add( Material.RECORD_11 );
    	Discs.add( Material.RECORD_3 );
    	Discs.add( Material.RECORD_4 );
    	Discs.add( Material.RECORD_5 );
    	Discs.add( Material.RECORD_6 );
    	Discs.add( Material.RECORD_7 );
    	Discs.add( Material.RECORD_8 );
    	Discs.add( Material.RECORD_9 );
    	
    	// Start playing a random one, but not the one we just played
    	int lastPlayed = this.getConfig().getInt("lastPlayed", 0 );
    	int index = lastPlayed;
    	do
    	{
    		Random generator = new Random( System.currentTimeMillis() );
    		index = generator.nextInt( Discs.size() );
    	} while( index == lastPlayed );
    	this.getConfig().set("lastPlayed", index );
    	this.saveConfig();
    	jb.setPlaying( Discs.get( index ) );
	}
	
	/**
	 * Process commands
	 * @param sender Player or console who sent the command
	 * @param command Command that was send
	 * @param label
	 * @param args Arguments
	 */
	public boolean onCommand( CommandSender sender, Command command, String label, String[] args )
	{
		// Check for valid args
		if ( args.length == 0 ) return false;
		
		// Play command
		if ( args[0].equalsIgnoreCase( "play" ) )
		{
			// Check permissions
			if ( !sender.hasPermission( "autodisc.play" ) )
			{
				sender.sendMessage( ChatColor.RED + "You don't have permission to do that!" );
				return true;
			}
			sender.sendMessage( "Force playing!" );
			this.stopTask();
			this.startTask();
			return true;
		}
		
		// Set command
		if ( args[0].equalsIgnoreCase( "set" ) )
		{
			// Check if the sender is a player or the server
			if ( !( sender instanceof Player ) )
			{
				sender.sendMessage( " Only players can use that command!" );
				return true;
			}
			Player player = ( Player ) sender;
			
			// Check permissions
			if ( !player.hasPermission( "autodisc.set" ) )
			{
				player.sendMessage( ChatColor.RED + "You don't have permission to do that!" );
				return true;
			}
			int x = player.getLocation().getBlockX();
			int y = player.getLocation().getBlockY();
			int z = player.getLocation().getBlockZ();
			this.getConfig().set( "coords.x", x );
			this.getConfig().set( "coords.y", y );
			this.getConfig().set( "coords.z", z );
			this.saveConfig();
			player.sendMessage( "Location Set (" + x + "," + y + "," + z +")" );
			return true;
		}
		
		// Set delay command
		if ( args[0].equalsIgnoreCase( "delay" ) )
		{
			// Check permissions
			if ( !sender.hasPermission( "autodisc.delay" ) )
			{
				sender.sendMessage( ChatColor.RED + "You don't have permission to do that!" );
				return true;
			}
			
			// Get number
			if ( args.length < 2 )
			{
				sender.sendMessage( ChatColor.RED + "You must enter a delay!" );
				return true;
			}
			Long delay = Long.parseLong( args[1] );
			this.getConfig().set( "delay", delay );
			this.saveConfig();
			this.stopTask();
			this.startTask();
			sender.sendMessage( "Delay Set!" );
			return true;
		}
		
		// Invalid argument
		return false;
	}
}